/* 
 * UDP server for Hantek / Tekway / Voltcraft / Protek / PCE oscilloscopes
 * 
 * contributors:
 *      
 *      Name                    Date        Changes
 *      David Beinder (yoq)     2012        
 * 
 *
 * This code is licenced under the GPL
 *
 */
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/socket.h>
#include <linux/net.h>
#include <net/sock.h>
#include <asm/processor.h>
#include <asm/uaccess.h>
#include <linux/in.h>

#include "ksocket.h"

#define BUFLEN 10500 //maximum packetsize that can be processed by dso.exe, according to <http://www.mikrocontroller.net/articles/Hantek_Protokoll#0x11_Virtuelle_Remote-Shell> this is 10'239Bytes 
#define PORT 9966

#ifndef MIN
#define MIN( a, b ) ((a)<(b)?(a):(b))
#endif

ksocket_t sockID = NULL;
char readBuf[BUFLEN];
int msgLength = 0;
struct sockaddr_in sender;
int addrLen = sizeof(sender);
void (*dataReceivedCB)(char *data, int bytes) = NULL;

static void readCallback(struct sock *sk, int len)
{
    //printk("udpserver: callback\n");
    msgLength = krecvfrom(sockID, readBuf, BUFLEN, 0, (struct sockaddr*)&sender, &addrLen);
    if(msgLength == -1)
    {
        printk("udpserver: error when reading message to buffer\n");
        return;
    }
    
    if(dataReceivedCB)
        dataReceivedCB(readBuf, msgLength);
    else
        printk("udpserver: data received, but no callback registered!");
}

static void udps_start(void)
{
    struct sockaddr_in listenAddr;
    
    sockID = ksocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (sockID == NULL)
        printk("udpserver: couldn't get socket\n");
        
    memset(&listenAddr, 0, sizeof(listenAddr));
    listenAddr.sin_family = AF_INET;
    listenAddr.sin_port = htons(PORT);
    listenAddr.sin_addr.s_addr = INADDR_ANY;
    
    sockID->sk->sk_data_ready = readCallback;
       
    if (kbind(sockID, (struct sockaddr*)&listenAddr, sizeof(listenAddr)) == -1)
        printk("udpserver: couldn't bind port\n");
    
    printk("udpserver: server running, port bound\n");
}

static void udps_stop(void)
{
    if(sockID != NULL)
        kclose(sockID);
}

static int udps_sendResponse(char *buffer, int length)
{
    int result = ksendto(sockID, buffer, length, 0, (struct sockaddr*)&sender, addrLen);
    
    if(result < 0)
    {
        printk("udpserver: sending response failed: %d\n", result);
        return 0;
    }
    
    return result;
}

//pass NULL to remove callback!
static void udps_setCallback(void (*recvCallback)(char *data, int bytes))
{
	dataReceivedCB = recvCallback;
}

static int udps_init(void)
{
	printk("udpserver: module loaded\n");
	return 0;
}

static void udps_exit(void)
{
    udps_stop();
	printk("udpserver: module unloaded\n");
}

module_init(udps_init);
module_exit(udps_exit);

EXPORT_SYMBOL(udps_start);
EXPORT_SYMBOL(udps_stop);
EXPORT_SYMBOL(udps_setCallback);
EXPORT_SYMBOL(udps_sendResponse);

MODULE_LICENSE("GPL");
